#include "bezier_drawer.hpp"

#include <exception>
#include <iostream>
#include <cmath>


vec2 coord_to_axis_convert(const vec2& p,float w,float h)
{
    const float L=150.0f;
    vec2 p_axis=p;
    p_axis.y*=-1.0f;
    p_axis*=L;
    p_axis.y+=h/2.0f;
    p_axis.x+=w/2.0f;
    return p_axis;
}

vec2 axis_to_coord_convert(const vec2& p,float w,float h)
{
    const float L=150.0f;
    vec2 p_coord=p;
    p_coord.x-=w/2.0f;
    p_coord.y-=h/2.0f;
    p_coord/=L;
    p_coord.y*=-1.0f;
    return p_coord;
}


bezier_drawer::bezier_drawer()
{
    data.coeff(0)=vec2(0.0f,0.0f);
    data.coeff(1)=vec2(1.0f/3.0f,1.0f);
    data.coeff(2)=vec2(2.0f/3.0f,1.0f);
    data.coeff(3)=vec2(1.0f,0.0f);
}

void bezier_drawer::draw_control_polygon_line(QPainter& painter,float w,float h) const
{
    const vec2 p0=coord_to_axis_convert(data.coeff(0),w,h);
    const vec2 p1=coord_to_axis_convert(data.coeff(1),w,h);
    const vec2 p2=coord_to_axis_convert(data.coeff(2),w,h);
    const vec2 p3=coord_to_axis_convert(data.coeff(3),w,h);

    painter.drawLine(p0.x,p0.y,p1.x,p1.y);
    painter.drawLine(p1.x,p1.y,p2.x,p2.y);
    painter.drawLine(p2.x,p2.y,p3.x,p3.y);
}

void bezier_drawer::draw_control_polygon_points(QPainter& painter,float w,float h) const
{
    const vec2 p0=coord_to_axis_convert(data.coeff(0),w,h);
    const vec2 p1=coord_to_axis_convert(data.coeff(1),w,h);
    const vec2 p2=coord_to_axis_convert(data.coeff(2),w,h);
    const vec2 p3=coord_to_axis_convert(data.coeff(3),w,h);

    painter.drawPoint(p0.x,p0.y);
    painter.drawPoint(p1.x,p1.y);
    painter.drawPoint(p2.x,p2.y);
    painter.drawPoint(p3.x,p3.y);
}

void bezier_drawer::draw_control_polygon_point(QPainter& painter,int vertex_selected,int w,int h) const
{
    if(vertex_selected<0 || vertex_selected>3)
        throw std::exception();
    const vec2 p0=coord_to_axis_convert(data.coeff(vertex_selected),w,h);
    painter.drawPoint(p0.x,p0.y);
}

void bezier_drawer::draw_bezier_line(QPainter& painter,float w,float h) const
{
    const int N=150;
    vec2 p_old=coord_to_axis_convert(data(0.0f),w,h);
    for(int k=1;k<N;++k)
    {
        const float s=static_cast<float>(k)/(N-1);
        const vec2 p=coord_to_axis_convert(data(s),w,h);
        painter.drawLine(p_old.x,p_old.y,p.x,p.y);
        p_old=p;
    }
}

int bezier_drawer::get_closest_vertex_control(int x,int y,float w,float h,float threshold) const
{
    const vec2 p_click=axis_to_coord_convert(vec2(x,y),w,h);

    int k_d_min=0;
    float d_min=norm(p_click-data.coeff(0));

    for(int k=1;k<=3;++k)
    {
        float dist=norm(p_click-data.coeff(k));
        if(dist<d_min)
        {
            k_d_min=k;
            d_min=dist;
        }
    }


    if(d_min<threshold)
        return k_d_min;
    else
        return -1;

}

void bezier_drawer::translate_vertex(int index,int x,int y)
{
    vec2 tr=vec2(x,-y)/150.0f;

    data.coeff(index)+=tr;

}
